@@ -1,10 +1,18 @@
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE DataKinds #-}
 {-# LANGUAGE FlexibleContexts #-}
+{-# LANGUAGE FlexibleInstances #-}
+{-# LANGUAGE OverloadedLabels #-}
+{-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE UndecidableInstances #-}
 
 module Web.Twitter.Conduit.Base
-       ( getResponse
+       ( ResponseBodyType (..)
+       , NoContent
+       , getResponse
        , call
        , call'
        , callWithResponse
@@ -23,71 +31,85 @@ module Web.Twitter.Conduit.Base
        ) where
 
 import Web.Twitter.Conduit.Cursor
-import Web.Twitter.Conduit.Parameters hiding (url)
 import Web.Twitter.Conduit.Request
+import Web.Twitter.Conduit.Request.Internal
 import Web.Twitter.Conduit.Response
 import Web.Twitter.Conduit.Types
 import Web.Twitter.Types.Lens
 
 import Control.Lens
-import Control.Monad.Base
+import Control.Monad (void)
 import Control.Monad.Catch (MonadThrow (..))
 import Control.Monad.IO.Class
 import Control.Monad.Trans.Resource (MonadResource, ResourceT, runResourceT)
 import Data.Aeson
 import Data.Aeson.Lens
 import Data.ByteString (ByteString)
+import Data.Coerce
 import qualified Data.Conduit as C
 import qualified Data.Conduit.Attoparsec as CA
 import qualified Data.Conduit.List as CL
 import qualified Data.Map as M
-import Data.Monoid
 import qualified Data.Text.Encoding as T
 import Network.HTTP.Client.MultipartFormData
 import qualified Network.HTTP.Conduit as HTTP
 import qualified Network.HTTP.Types as HT
-import Unsafe.Coerce
 import Web.Authenticate.OAuth (signOAuth)
 
+#if __GLASGOW_HASKELL__ < 804
+import Data.Monoid
+#endif
+
 makeRequest :: APIRequest apiName responseType
             -> IO HTTP.Request
-makeRequest (APIRequestGet u pa) = makeRequest' "GET" u (makeSimpleQuery pa)
-makeRequest (APIRequestPost u pa) = makeRequest' "POST" u (makeSimpleQuery pa)
-makeRequest (APIRequestPostMultipart u param prt) =
-    formDataBody body =<< makeRequest' "POST" u []
+makeRequest (APIRequest m u pa) = makeRequest' m u (makeSimpleQuery pa)
+makeRequest (APIRequestMultipart m u param prt) =
+    formDataBody body =<< makeRequest' m u []
   where
     body = prt ++ partParam
     partParam = Prelude.map (uncurry partBS . over _1 T.decodeUtf8) (makeSimpleQuery param)
+makeRequest (APIRequestJSON m u param body) = do
+    req <- makeRequest' m u (makeSimpleQuery param)
+    return $
+        req
+        { HTTP.requestBody = HTTP.RequestBodyLBS $ encode body
+        , HTTP.requestHeaders = ("Content-Type", "application/json") : HTTP.requestHeaders req
+        }
 
 makeRequest' :: HT.Method -- ^ HTTP request method (GET or POST)
              -> String -- ^ API Resource URL
              -> HT.SimpleQuery -- ^ Query
              -> IO HTTP.Request
 makeRequest' m url query = do
-#if MIN_VERSION_http_client(0,4,30)
     req <- HTTP.parseRequest url
-#else
-    req <- HTTP.parseUrl url
-#endif
     let addParams =
             if m == "POST"
             then HTTP.urlEncodedBody query
             else \r -> r { HTTP.queryString = HT.renderSimpleQuery False query }
-    return $ addParams $ req { HTTP.method = m
-#if !MIN_VERSION_http_client(0,4,30)
-                             , HTTP.checkStatus = \_ _ _ -> Nothing
-#endif
-                             }
+    return $ addParams $ req { HTTP.method = m }
+
+class ResponseBodyType a where
+    parseResponseBody ::
+           Response (C.ConduitM () ByteString (ResourceT IO) ())
+        -> ResourceT IO (Response a)
+
+type NoContent = ()
+instance ResponseBodyType NoContent where
+    parseResponseBody res =
+        case responseStatus res of
+            st | st == HT.status204 -> return $ void res
+            _ -> do
+                body <- C.runConduit $ responseBody res C..| sinkJSON
+                throwM $ TwitterStatusError (responseStatus res) (responseHeaders res) body
+
+instance {-# OVERLAPPABLE #-} FromJSON a => ResponseBodyType a where
+    parseResponseBody = getValueOrThrow
 
 getResponse :: MonadResource m
             => TWInfo
             -> HTTP.Manager
             -> HTTP.Request
-#if MIN_VERSION_http_conduit(2,3,0)
             -> m (Response (C.ConduitM () ByteString m ()))
-#else
-            -> m (Response (C.ResumableSource m ByteString))
-#endif
 getResponse TWInfo{..} mgr req = do
     signedReq <- signOAuth (twOAuth twToken) (twCredential twToken) $ req { HTTP.proxy = twProxy }
     res <- HTTP.http signedReq mgr
@@ -101,19 +123,11 @@ endpoint :: String
 endpoint = "https://api.twitter.com/1.1/"
 
 getValue ::
-#if MIN_VERSION_http_conduit(2,3,0)
             Response (C.ConduitM () ByteString (ResourceT IO) ())
-#else
-            Response (C.ResumableSource (ResourceT IO) ByteString)
-#endif
          -> ResourceT IO (Response Value)
 getValue res = do
     value <-
-#if MIN_VERSION_http_conduit(2,3,0)
       C.runConduit $ responseBody res C..| sinkJSON
-#else
-      responseBody res C.$$+- sinkJSON
-#endif
     return $ res { responseBody = value }
 
 checkResponse :: Response Value
@@ -134,11 +148,7 @@ checkResponse Response{..} =
     sci = HT.statusCode responseStatus
 
 getValueOrThrow :: FromJSON a
-#if MIN_VERSION_http_conduit(2,3,0)
                 => Response (C.ConduitM () ByteString (ResourceT IO) ())
-#else
-                => Response (C.ResumableSource (ResourceT IO) ByteString)
-#endif
                 -> ResourceT IO (Response a)
 getValueOrThrow res = do
     res' <- getValue res
@@ -161,7 +171,7 @@ getValueOrThrow res = do
 --
 -- If you need raw JSON value which is parsed by <http://hackage.haskell.org/package/aeson aeson>,
 -- use 'call'' to obtain it.
-call :: FromJSON responseType
+call :: ResponseBodyType responseType
      => TWInfo -- ^ Twitter Setting
      -> HTTP.Manager
      -> APIRequest apiName responseType
@@ -171,7 +181,7 @@ call = call'
 -- | Perform an 'APIRequest' and then provide the response.
 -- The response of this function is not restrict to @responseType@,
 -- so you can choose an arbitrarily type of FromJSON instances.
-call' :: FromJSON value
+call' :: ResponseBodyType value
       => TWInfo -- ^ Twitter Setting
       -> HTTP.Manager
       -> APIRequest apiName responseType
@@ -188,7 +198,7 @@ call' info mgr req = responseBody `fmap` callWithResponse' info mgr req
 -- 'print' $ 'responseHeaders' res
 -- 'print' $ 'responseBody' res
 -- @
-callWithResponse :: FromJSON responseType
+callWithResponse :: ResponseBodyType responseType
                  => TWInfo -- ^ Twitter Setting
                  -> HTTP.Manager
                  -> APIRequest apiName responseType
@@ -207,7 +217,7 @@ callWithResponse = callWithResponse'
 -- 'print' $ 'responseHeaders' res
 -- 'print' $ 'responseBody' (res :: Value)
 -- @
-callWithResponse' :: FromJSON value
+callWithResponse' :: ResponseBodyType value
                   => TWInfo
                   -> HTTP.Manager
                   -> APIRequest apiName responseType
@@ -215,7 +225,7 @@ callWithResponse' :: FromJSON value
 callWithResponse' info mgr req =
     runResourceT $ do
         res <- getResponse info mgr =<< liftIO (makeRequest req)
-        getValueOrThrow res
+        parseResponseBody res
 
 -- | A wrapper function to perform multiple API request with changing @max_id@ parameter.
 --
@@ -223,12 +233,12 @@ callWithResponse' info mgr req =
 sourceWithMaxId :: ( MonadIO m
                    , FromJSON responseType
                    , AsStatus responseType
-                   , HasMaxIdParam (APIRequest apiName [responseType])
+                   , HasParam "max_id" Integer supports
                    )
                 => TWInfo -- ^ Twitter Setting
                 -> HTTP.Manager
-                -> APIRequest apiName [responseType]
-                -> C.Source m responseType
+                -> APIRequest supports [responseType]
+                -> C.ConduitT () responseType m ()
 sourceWithMaxId info mgr = loop
   where
     loop req = do
@@ -236,7 +246,7 @@ sourceWithMaxId info mgr = loop
         case getMinId res of
             Just mid -> do
                 CL.sourceList res
-                loop $ req & maxId ?~ mid - 1
+                loop $ req & #max_id ?~ mid - 1
             Nothing -> CL.sourceList res
     getMinId = minimumOf (traverse . status_id)
 
@@ -246,22 +256,21 @@ sourceWithMaxId info mgr = loop
 --
 -- This function cooperate with instances of 'HasMaxIdParam'.
 sourceWithMaxId' :: ( MonadIO m
-                    , HasMaxIdParam (APIRequest apiName [responseType])
+                    , HasParam "max_id" Integer supports
                     )
                  => TWInfo -- ^ Twitter Setting
                  -> HTTP.Manager
-                 -> APIRequest apiName [responseType]
-                 -> C.Source m Value
+                 -> APIRequest supports [responseType]
+                 -> C.ConduitT () Value m ()
 sourceWithMaxId' info mgr = loop
   where
     loop req = do
-        res <- liftIO $ call' info mgr req
-        case getMinId res of
+        (res :: [Value]) <- liftIO $ call' info mgr req
+        case minimumOf (traverse . key "id" . _Integer) res of
             Just mid -> do
                 CL.sourceList res
-                loop $ req & maxId ?~ mid - 1
+                loop $ req & #max_id ?~ mid - 1
             Nothing -> CL.sourceList res
-    getMinId = minimumOf (traverse . key "id" . _Integer)
 
 -- | A wrapper function to perform multiple API request with changing @cursor@ parameter.
 --
@@ -269,17 +278,18 @@ sourceWithMaxId' info mgr = loop
 sourceWithCursor :: ( MonadIO m
                     , FromJSON responseType
                     , CursorKey ck
-                    , HasCursorParam (APIRequest apiName (WithCursor ck responseType))
+                    , HasParam "cursor" Integer supports
                     )
                  => TWInfo -- ^ Twitter Setting
                  -> HTTP.Manager
-                 -> APIRequest apiName (WithCursor ck responseType)
-                 -> C.Source m responseType
-sourceWithCursor info mgr req = loop (-1)
+                 -> APIRequest supports (WithCursor Integer ck responseType)
+                 -> C.ConduitT () responseType m ()
+sourceWithCursor info mgr req = loop (Just (-1))
   where
-    loop 0 = CL.sourceNull
-    loop cur = do
-        res <- liftIO $ call info mgr $ req & cursor ?~ cur
+    loop Nothing = CL.sourceNull
+    loop (Just 0) = CL.sourceNull
+    loop (Just cur) = do
+        res <- liftIO $ call info mgr $ req & #cursor ?~ cur
         CL.sourceList $ contents res
         loop $ nextCursor res
 
@@ -290,20 +300,21 @@ sourceWithCursor info mgr req = loop (-1)
 -- This function cooperate with instances of 'HasCursorParam'.
 sourceWithCursor' :: ( MonadIO m
                      , CursorKey ck
-                     , HasCursorParam (APIRequest apiName (WithCursor ck responseType))
+                     , HasParam "cursor" Integer supports
                      )
                   => TWInfo -- ^ Twitter Setting
                   -> HTTP.Manager
-                  -> APIRequest apiName (WithCursor ck responseType)
-                  -> C.Source m Value
-sourceWithCursor' info mgr req = loop (-1)
+                  -> APIRequest supports (WithCursor Integer ck responseType)
+                  -> C.ConduitT () Value m ()
+sourceWithCursor' info mgr req = loop (Just (-1))
   where
-    relax :: APIRequest apiName (WithCursor ck responseType)
-          -> APIRequest apiName (WithCursor ck Value)
-    relax = unsafeCoerce
-    loop 0 = CL.sourceNull
-    loop cur = do
-        res <- liftIO $ call info mgr $ relax $ req & cursor ?~ cur
+    relax :: APIRequest apiName (WithCursor Integer ck responseType)
+          -> APIRequest apiName (WithCursor Integer ck Value)
+    relax = coerce
+    loop Nothing = CL.sourceNull
+    loop (Just 0) = CL.sourceNull
+    loop (Just cur) = do
+        res <- liftIO $ call info mgr $ relax $ req & #cursor ?~ cur
         CL.sourceList $ contents res
         loop $ nextCursor res
 
@@ -313,8 +324,8 @@ sourceWithSearchResult :: ( MonadIO m
                           )
                        => TWInfo -- ^ Twitter Setting
                        -> HTTP.Manager
-                       -> APIRequest apiName (SearchResult [responseType])
-                       -> m (SearchResult (C.Source m responseType))
+                       -> APIRequest supports (SearchResult [responseType])
+                       -> m (SearchResult (C.ConduitT () responseType m ()))
 sourceWithSearchResult info mgr req = do
     res <- liftIO $ call info mgr req
     let body = CL.sourceList (res ^. searchResultStatuses) <>
@@ -335,8 +346,8 @@ sourceWithSearchResult' :: ( MonadIO m
                            )
                         => TWInfo -- ^ Twitter Setting
                         -> HTTP.Manager
-                        -> APIRequest apiName (SearchResult [responseType])
-                        -> m (SearchResult (C.Source m Value))
+                        -> APIRequest supports (SearchResult [responseType])
+                        -> m (SearchResult (C.ConduitT () Value m ()))
 sourceWithSearchResult' info mgr req = do
     res <- liftIO $ call info mgr $ relax req
     let body = CL.sourceList (res ^. searchResultStatuses) <>
@@ -346,7 +357,7 @@ sourceWithSearchResult' info mgr req = do
     origQueryMap = req ^. params . to M.fromList
     relax :: APIRequest apiName (SearchResult [responseType])
           -> APIRequest apiName (SearchResult [Value])
-    relax = unsafeCoerce
+    relax = coerce
     loop Nothing = CL.sourceNull
     loop (Just nextResultsStr) = do
         let nextResults = nextResultsStr & HT.parseSimpleQuery . T.encodeUtf8 & traversed . _2 %~ (PVString . T.decodeUtf8)
@@ -356,12 +367,12 @@ sourceWithSearchResult' info mgr req = do
         loop $ res ^. searchResultSearchMetadata . searchMetadataNextResults
 
 sinkJSON :: ( MonadThrow m
-            ) => C.Consumer ByteString m Value
+            ) => C.ConduitT ByteString o m Value
 sinkJSON = CA.sinkParser json
 
 sinkFromJSON :: ( FromJSON a
                 , MonadThrow m
-                ) => C.Consumer ByteString m a
+                ) => C.ConduitT ByteString o m a
 sinkFromJSON = do
     v <- sinkJSON
     case fromJSON v of
